'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.default = reduce;

var _lodash = require('lodash');

var _client = require('@slack/client');

var _utils = require('./utils');

/**
 * Handles the messages subtree for channels, ims, and groups.
 *
 * @param  {Object} state    The starting state
 * @param  {String} {type    The type of message action
 * @param  {Object} message} The message object from the socket
 * @return {Object}          The modified state
 */
function reduce(state, _ref) {
  let type = _ref.type;
  let message = _ref.message;


  if (type.startsWith(_client.RTM_EVENTS.MESSAGE)) {
    // Since this reducer is shared across channels, groups, and ims, we need to
    // first check that this message pertains to us.
    let channel = state[message.channel];
    if (!channel) return state;

    switch ((0, _utils.subtypeFromActionType)(type)) {
      case _utils.DEFAULT_MESSAGE_ACTION:
        return addMessage(state, message);
      case _client.RTM_MESSAGE_SUBTYPES.MESSAGE_CHANGED:
        return editMessage(state, message);
      case _client.RTM_MESSAGE_SUBTYPES.MESSAGE_DELETED:
        return deleteMessage(state, message);
      case _client.RTM_MESSAGE_SUBTYPES.CHANNEL_JOIN:
        return joinChannelMessage(state, message);
      case _client.RTM_MESSAGE_SUBTYPES.CHANNEL_LEAVE:
        return leaveChannelMessage(state, message);
    }
  }

  switch (type) {
    case _client.RTM_EVENTS.CHANNEL_MARKED:
      return markChannel(state, message);
    case _client.RTM_EVENTS.REACTION_ADDED:
      return addReaction(state, message);
    case _client.RTM_EVENTS.REACTION_REMOVED:
      return removeReaction(state, message);
    default:
      return state;
  }
}

function addMessage(state, message) {
  let channel = state[message.channel];
  let isNewestMessage = message.ts > channel.max_ts && !message.hidden;

  return _extends({}, state, {
    [channel.id]: _extends({}, channel, {
      max_ts: isNewestMessage ? message.ts : channel.max_ts,
      unread_count: isNewestMessage ? channel.unread_count + 1 : channel.unread_count,
      messages: _extends({}, channel.messages, {
        [message.ts]: message
      })
    })
  });
}

function editMessage(state, message) {
  let channel = state[message.channel];
  let editedMessage = message.message;

  return _extends({}, state, {
    [channel.id]: _extends({}, channel, {
      messages: _extends({}, channel.messages, {
        [editedMessage.ts]: editedMessage,
        [message.ts]: message
      })
    })
  });
}

function deleteMessage(state, message) {
  let channel = state[message.channel];

  return _extends({}, state, {
    [channel.id]: _extends({}, channel, {
      messages: _extends({}, (0, _lodash.omit)(channel.messages, message.deleted_ts), {
        [message.ts]: message
      })
    })
  });
}

function joinChannelMessage(state, message) {
  let channel = state[message.channel];

  return _extends({}, state, {
    [channel.id]: _extends({}, channel, {
      members: [...channel.members, message.user],
      messages: _extends({}, channel.messages, {
        [message.ts]: message
      })
    })
  });
}

function leaveChannelMessage(state, message) {
  let channel = state[message.channel];

  return _extends({}, state, {
    [channel.id]: _extends({}, channel, {
      members: [...channel.members.slice(0, channel.members.indexOf(message.user)), ...channel.members.slice(channel.members.indexOf(message.user) + 1)],
      messages: _extends({}, channel.messages, {
        [message.ts]: message
      })
    })
  });
}

function markChannel(state, message) {
  let channel = state[message.channel];
  if (!channel) return state;

  let unreadCount = 0;

  (0, _lodash.forEachRight)(channel.messages, m => {
    if (m.ts > message.ts && !m.hidden) {
      unreadCount++;
      return true;
    }
    return false;
  });

  return _extends({}, state, {
    [channel.id]: _extends({}, channel, {
      last_read: message.ts,
      unread_count: unreadCount
    })
  });
}

function addReaction(state, _ref2) {
  let item = _ref2.item;
  let reaction = _ref2.reaction;
  let user = _ref2.user;

  let channel = state[item.channel];
  if (!channel) return state;

  let message = channel.messages[item.ts];
  if (!message) return state;

  let reactionIndex = (0, _lodash.findIndex)(message.reactions, { name: reaction });
  let reactionToAddOrIncrement;

  if (reactionIndex !== -1) {
    let existingReaction = message.reactions[reactionIndex];

    reactionToAddOrIncrement = _extends({}, existingReaction, {
      count: existingReaction.count + 1,
      users: [...existingReaction.users, user]
    });
  } else {
    reactionToAddOrIncrement = {
      name: reaction,
      users: [user],
      count: 1
    };
  }

  return _extends({}, state, {
    [channel.id]: _extends({}, channel, {
      messages: _extends({}, channel.messages, {
        [message.ts]: _extends({}, message, {
          reactions: [...(message.reactions || []).slice(0, reactionIndex), reactionToAddOrIncrement, ...(message.reactions || []).slice(reactionIndex + 1)]
        })
      })
    })
  });
}

function removeReaction(state, _ref3) {
  let item = _ref3.item;
  let reaction = _ref3.reaction;
  let user = _ref3.user;

  let channel = state[item.channel];
  if (!channel) return state;

  let message = channel.messages[item.ts];
  if (!message) return state;

  let reactionIndex = (0, _lodash.findIndex)(message.reactions, { name: reaction });
  if (reactionIndex === -1) return state;

  let existingReaction = message.reactions[reactionIndex];
  let reactionToRemoveOrDecrement = existingReaction.count > 1 ? [_extends({}, existingReaction, {
    count: existingReaction.count - 1,
    users: (0, _lodash.without)(existingReaction.users, user)
  })] : [];

  return _extends({}, state, {
    [channel.id]: _extends({}, channel, {
      messages: _extends({}, channel.messages, {
        [message.ts]: _extends({}, message, {
          reactions: [...message.reactions.slice(0, reactionIndex), ...reactionToRemoveOrDecrement, ...message.reactions.slice(reactionIndex + 1)]
        })
      })
    })
  });
}