'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _lodash = require('lodash');

var _redux = require('redux');

var _helpers = require('./reducers/helpers');

var _reduxThunk = require('redux-thunk');

var _reduxThunk2 = _interopRequireDefault(_reduxThunk);

var _actions = require('./actions');

var _actions2 = _interopRequireDefault(_actions);

var _reducers = require('./reducers');

var reducers = _interopRequireWildcard(_reducers);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

class SlackReduxStore {

  /**
   * Creates a new instance of `SlackReduxStore`.
   *
   * @param  {Array} storeEnhancers An array of store enhancers to apply
   */
  constructor() {
    let storeEnhancers = arguments.length <= 0 || arguments[0] === undefined ? [] : arguments[0];

    let initialState = {};
    let toCompose = [(0, _redux.applyMiddleware)(_reduxThunk2.default), ...storeEnhancers];

    this.store = (0, _redux.createStore)((0, _redux.combineReducers)(reducers.default), initialState, (0, _redux.compose)(...toCompose));
  }

  /**
   * Hydrates the store with the `rtm.start` response.
   *
   * @param  {Object} payload The full `rtm.start` response
   */
  cacheRtmStart(payload) {
    this.dispatch({
      type: _helpers.RTM_START,
      data: payload
    });
  }

  /**
   * Handles a real-time message by dispatching an action to the store.
   *
   * @param  {String} userId  The ID of the active user
   * @param  {String} teamId  The ID of the active team
   * @param  {String} type    The type of message
   * @param  {Object} message The message from the socket
   */
  handleRtmMessage(userId, teamId, type, message) {
    let action = (0, _actions2.default)(type, message, userId, teamId);
    this.dispatch(action);
  }

  /**
   * Returns the underlying Redux store.
   *
   * @return {Object}  The result of Redux `createStore`
   */
  getStore() {
    return this.store;
  }

  /**
   * Dispatches an action to the underlying store.
   *
   * @param  {Object} action The action to dispatch
   */
  dispatch(action) {
    this.store.dispatch(action);
  }

  /**
   * Subscribes a listener to state changes from the store.
   *
   * @param  {Function} listener The listener to subscribe
   * @return {Function}          A method that will unsubscribe it
   */
  subscribe(listener) {
    return this.store.subscribe(listener);
  }

  /**
   * Returns the current state of the store.
   *
   * @return {Object}  The state tree
   */
  getState() {
    return this.store.getState();
  }

  getUserById(userId) {
    return this.getState().users[userId];
  }

  getUserByName(name) {
    return (0, _lodash.find)(this.getState().users, user => user.name === name);
  }

  getUserByEmail(email) {
    return (0, _lodash.find)(this.getState().users, user => user.email === email);
  }

  getChannelById(channelId) {
    return this.getState().channels[channelId];
  }

  getChannelByName(name) {
    return (0, _lodash.find)(this.getState().channels, channel => channel.name === name);
  }

  getGroupById(groupId) {
    return this.getState().groups[groupId];
  }

  getGroupByName(name) {
    return (0, _lodash.find)(this.getState().groups, group => group.name === name);
  }

  getDMById(dmId) {
    return this.getState().ims[dmId];
  }

  getDMByName(name) {
    return (0, _lodash.find)(this.getState().ims, dm => dm.name === name);
  }

  getBotById(botId) {
    return this.getState().bots[botId];
  }

  getBotByName(name) {
    return (0, _lodash.find)(this.getState().bots, bot => bot.name === name);
  }

  getTeamById(teamId) {
    return this.getState().teams[teamId];
  }

  getUnreadCount() {
    let getUnreads = collection => (0, _lodash.reduce)(collection, (total, c) => total + c.unread_count, 0);

    return getUnreads(this.getState().channels) + getUnreads(this.getState().ims) + getUnreads(this.getState().groups);
  }

  getChannelGroupOrDMById(objId) {
    let ret;
    let firstChar = objId.substring(0, 1);

    if (firstChar === 'C') {
      ret = this.getChannelById(objId);
    } else if (firstChar === 'G') {
      ret = this.getGroupById(objId);
    } else if (firstChar === 'D') {
      ret = this.getDMById(objId);
    }

    return ret;
  }

  getChannelOrGroupByName(name) {
    let channel = this.getChannelByName(name);
    return channel || this.getGroupByName(name);
  }
}
exports.default = SlackReduxStore;